// FTSFlipsideViewController.m
// FTSFlipsideViewController displays search results in a UIWebView
#import "FTSFlipsideViewController.h"

// private FTSFlipsideViewController interface 
@interface FTSFlipsideViewController () <UIWebViewDelegate>
@property (weak, nonatomic) IBOutlet UIWebView *webView;

@end // FTSFlipsideViewController private interface

@implementation FTSFlipsideViewController

// requests Twitter search results page when flipside view loads
- (void)viewDidLoad
{
   [super viewDidLoad]; // call superclass's viewDidLoad method
	
   // set FTSFlipsideViewController as the UIWebViewDelegate
   [self.webView setDelegate:self];
   
   // tell the UIWebView to request the searchURL
   [self.webView
      loadRequest:[NSURLRequest requestWithURL:self.searchURL]];
} // end method viewDidLoad

// returns to main view when user touches the Done button
- (IBAction)done:(id)sender
{
   // remove activity indicator from the status bar when user touches Done
   [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
   [self.webView stopLoading]; // stop loading web page

   // ensure that delegate implements flipsideViewControllerDidFinish:
   if ([self.delegate
      respondsToSelector:@selector(flipsideViewControllerDidFinish:)])
   {
      [self.delegate flipsideViewControllerDidFinish:self];
   } // end if
} // end method done:

// called when the UIWebView starts loading requested page
- (void)webViewDidStartLoad:(UIWebView *)webView
{
   // show activity indicator in the status bar when searchURL requested
   [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
} // end method webViewDidStartLoad:

// called when the UIWebView finishes loading requested page
- (void)webViewDidFinishLoad:(UIWebView *)webView
{
   // remove activity indicator from the status bar when results loaded
   [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
} // end method webViewDidStartLoad:

@end // FTSFlipsideViewController implementation


/**************************************************************************
 * (C) Copyright 2013 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code. You must attribute the code to      *
 * Deitel & Associates, Inc. and reference the book's web page at         *
 * www.deitel.com/books/iosfp2/. If you have any questions, please email  *
 * at deitel@deitel.com.                                                  *
 *************************************************************************/
