// TCViewController.m
// Implementation of class TCViewController
#import "TCViewController.h"

@interface TCViewController () // private interface of TCViewController

// properties for programmatically interacting with GUI components
@property (weak, nonatomic) IBOutlet UILabel *billTotalLabel;
@property (weak, nonatomic) IBOutlet UILabel *tip10Label;
@property (weak, nonatomic) IBOutlet UILabel *tip15Label;
@property (weak, nonatomic) IBOutlet UILabel *tip20Label;
@property (weak, nonatomic) IBOutlet UILabel *total10Label;
@property (weak, nonatomic) IBOutlet UILabel *total15Label;
@property (weak, nonatomic) IBOutlet UILabel *total20Label;
@property (weak, nonatomic) IBOutlet UISlider *customTipPercentageSlider;
@property (weak, nonatomic) IBOutlet UILabel *customTipPercentageLabel;
@property (weak, nonatomic) IBOutlet UILabel *customTipLabel;
@property (weak, nonatomic) IBOutlet UILabel *customTotalLabel;
@property (weak, nonatomic) IBOutlet UITextField *inputTextField;

@end // end the "private" interface for class TCViewController

// begin the implementation of class TCViewController
@implementation TCViewController

// display the number keypad when the view loads
- (void)viewDidLoad
{
   [super viewDidLoad]; // calls superclass's version of method
   
   // select inputTextField so its keypad displays when the app loads
   [self.inputTextField becomeFirstResponder];
} // end method viewDidLoad

// called when the user touches a key or button
- (IBAction)calculateTip:(id)sender
{
   // static local variables used only in this method
   static NSDecimalNumber *decimalNumber100; // 100 as a decimal
   static NSDecimalNumber *decimalNumber10Percent; // 0.10 as a decimal
   static NSDecimalNumber *decimalNumber15Percent; // 0.15 as a decimal
   static NSDecimalNumber *decimalNumber20Percent; // 0.20 as a decimal
   
   // variable used by dispatch_once to ensure block executes once
   static dispatch_once_t onceToken;
   
   // use Grand Central Dispatch to execute a block only once 
   dispatch_once(&onceToken,
      ^{
         decimalNumber100 =
            [NSDecimalNumber decimalNumberWithString:@"100.0"];
         decimalNumber10Percent =
            [NSDecimalNumber decimalNumberWithString:@"0.10"];
         decimalNumber15Percent =
            [NSDecimalNumber decimalNumberWithString:@"0.15"];
         decimalNumber20Percent =
            [NSDecimalNumber decimalNumberWithString:@"0.20"];
      } // end block
   ); // end call to dispatch_once
                 
   NSString *inputString = self.inputTextField.text; // get user input
   
   // retrieve the slider value and convert it to an NSDecimalNumber
   int sliderValue = (int) self.customTipPercentageSlider.value;
   NSDecimalNumber *decimalNumberSliderValue = [NSDecimalNumber
      decimalNumberWithMantissa:sliderValue exponent:0 isNegative:NO];

   // divide decimalNumberSliderValue by decimalNumber100 to get tip %
   NSDecimalNumber *decimalNumberCustomPercent = [decimalNumberSliderValue
      decimalNumberByDividingBy:decimalNumber100];
   
   // determine whether customTipPercentageSlider generated the event
   if (sender == self.customTipPercentageSlider)
   {
      // slider value changed so update the Label with the new percentage
      self.customTipPercentageLabel.text = [NSNumberFormatter
         localizedStringFromNumber:decimalNumberCustomPercent
         numberStyle:NSNumberFormatterPercentStyle];
   } // end if
   
   if ([inputString length] != 0) // if inputString is not empty
   {  
      // convert inputString to an NSDecimalNumber and divide it by 100
      NSDecimalNumber *decimalNumberTotal =
         [[NSDecimalNumber decimalNumberWithString:inputString]
            decimalNumberByDividingBy:decimalNumber100];
      
      // determine whether inputTextField generated the event
      if (sender == self.inputTextField)
      {
         // update billTotalInputLabel with the properly formatted number
         self.billTotalLabel.text =
            [self currencyStringWithNumber:decimalNumberTotal];
         
         // calculate the tips for 10, 15 and 20%
         NSDecimalNumber *decimalNumber10Tip = [decimalNumberTotal
            decimalNumberByMultiplyingBy:decimalNumber10Percent];
         NSDecimalNumber *decimalNumber15Tip = [decimalNumberTotal
            decimalNumberByMultiplyingBy:decimalNumber15Percent];
         NSDecimalNumber *decimalNumber20Tip = [decimalNumberTotal
            decimalNumberByMultiplyingBy:decimalNumber20Percent];
         
         // display the values for the 10, 15 and 20% tip labels
         self.tip10Label.text =
            [self currencyStringWithNumber:decimalNumber10Tip];
         self.tip15Label.text =
            [self currencyStringWithNumber:decimalNumber15Tip];
         self.tip20Label.text =
            [self currencyStringWithNumber:decimalNumber20Tip];
         
         // display the values for the 10, 15 and 20% total labels
         self.total10Label.text = [self currencyStringWithNumber:
            [decimalNumberTotal decimalNumberByAdding:decimalNumber10Tip]];
         self.total15Label.text = [self currencyStringWithNumber:
            [decimalNumberTotal decimalNumberByAdding:decimalNumber15Tip]];
         self.total20Label.text = [self currencyStringWithNumber:
            [decimalNumberTotal decimalNumberByAdding:decimalNumber20Tip]];
      } // end if
      
      // calculate customTip amount
      NSDecimalNumber *decimaNumberCustomTip = [decimalNumberTotal
         decimalNumberByMultiplyingBy:decimalNumberCustomPercent];
      
      // display the custom tip amount and custom total
      self.customTipLabel.text =
         [self currencyStringWithNumber:decimaNumberCustomTip];
      self.customTotalLabel.text = [self currencyStringWithNumber:
         [decimalNumberTotal decimalNumberByAdding:decimaNumberCustomTip]];
   } // end if
   else // clear all Labels
   {
      self.billTotalLabel.text = @""; // clear billTotalInputLabel
      self.tip10Label.text = @""; // clear tip10Label
      self.tip15Label.text = @""; // clear tip15Label
      self.tip20Label.text = @""; // clear tip20Label
      self.total10Label.text = @""; // clear total10Label
      self.total15Label.text = @""; // clear total15Label
      self.total20Label.text = @""; // clear total20Label
      self.customTipLabel.text = @""; // clear customTipLabel
      self.customTotalLabel.text = @""; // clear customTotalLabel
   } // end else
} // end method calculateTip:

// convert a float value to localized currency string
- (NSString *)currencyStringWithNumber:(NSNumber *)number
{
   return [NSNumberFormatter localizedStringFromNumber:number
      numberStyle:NSNumberFormatterCurrencyStyle];
} // end class method currencyStringWithFloat:

@end

/**************************************************************************
 * (C) Copyright 2013 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code. you must attribute the code to      *
 * Deitel & Associates, Inc. and reference the book's web page at         *
 * www.deitel.com/books/iosfp2/. If you have any questions, please email  *
 * at deitel@deitel.com.                                                  *
 *************************************************************************/
